"use strict";

Object.defineProperty(exports, "t", {
    value: true
});

exports.default = exports.LinkNode = exports.LinkListIterator = void 0;

var _Base = _interopRequireDefault(require("./Base"));

var _ContainerBase = require("../ContainerBase");

function _interopRequireDefault(t) {
    return t && t.t ? t : {
        default: t
    };
}

class LinkNode {
    constructor(t) {
        this.value = undefined;
        this.pre = undefined;
        this.next = undefined;
        this.value = t;
    }
}

exports.LinkNode = LinkNode;

class LinkListIterator extends _ContainerBase.ContainerIterator {
    constructor(t, i, s) {
        super(s);
        this.I = t;
        this.L = i;
        if (this.iteratorType === 0) {
            this.pre = function() {
                if (this.I.pre === this.L) {
                    throw new RangeError("LinkList iterator access denied!");
                }
                this.I = this.I.pre;
                return this;
            };
            this.next = function() {
                if (this.I === this.L) {
                    throw new RangeError("LinkList iterator access denied!");
                }
                this.I = this.I.next;
                return this;
            };
        } else {
            this.pre = function() {
                if (this.I.next === this.L) {
                    throw new RangeError("LinkList iterator access denied!");
                }
                this.I = this.I.next;
                return this;
            };
            this.next = function() {
                if (this.I === this.L) {
                    throw new RangeError("LinkList iterator access denied!");
                }
                this.I = this.I.pre;
                return this;
            };
        }
    }
    get pointer() {
        if (this.I === this.L) {
            throw new RangeError("LinkList iterator access denied!");
        }
        return this.I.value;
    }
    set pointer(t) {
        if (this.I === this.L) {
            throw new RangeError("LinkList iterator access denied!");
        }
        this.I.value = t;
    }
    equals(t) {
        return this.I === t.I;
    }
    copy() {
        return new LinkListIterator(this.I, this.L, this.iteratorType);
    }
}

exports.LinkListIterator = LinkListIterator;

class LinkList extends _Base.default {
    constructor(t = []) {
        super();
        this.L = new LinkNode;
        this.C = undefined;
        this.F = undefined;
        t.forEach((t => this.pushBack(t)));
    }
    clear() {
        this.o = 0;
        this.C = this.F = undefined;
        this.L.pre = this.L.next = undefined;
    }
    begin() {
        return new LinkListIterator(this.C || this.L, this.L);
    }
    end() {
        return new LinkListIterator(this.L, this.L);
    }
    rBegin() {
        return new LinkListIterator(this.F || this.L, this.L, 1);
    }
    rEnd() {
        return new LinkListIterator(this.L, this.L, 1);
    }
    front() {
        return this.C ? this.C.value : undefined;
    }
    back() {
        return this.F ? this.F.value : undefined;
    }
    forEach(t) {
        if (!this.o) return;
        let i = this.C;
        let s = 0;
        while (i !== this.L) {
            t(i.value, s++);
            i = i.next;
        }
    }
    getElementByPos(t) {
        if (t < 0 || t > this.o - 1) {
            throw new RangeError;
        }
        let i = this.C;
        while (t--) {
            i = i.next;
        }
        return i.value;
    }
    eraseElementByPos(t) {
        if (t < 0 || t > this.o - 1) {
            throw new RangeError;
        }
        if (t === 0) this.popFront(); else if (t === this.o - 1) this.popBack(); else {
            let i = this.C;
            while (t--) {
                i = i.next;
            }
            i = i;
            const s = i.pre;
            const e = i.next;
            e.pre = s;
            s.next = e;
            this.o -= 1;
        }
    }
    eraseElementByValue(t) {
        while (this.C && this.C.value === t) this.popFront();
        while (this.F && this.F.value === t) this.popBack();
        if (!this.C) return;
        let i = this.C;
        while (i !== this.L) {
            if (i.value === t) {
                const t = i.pre;
                const s = i.next;
                if (s) s.pre = t;
                if (t) t.next = s;
                this.o -= 1;
            }
            i = i.next;
        }
    }
    eraseElementByIterator(t) {
        const i = t.I;
        if (i === this.L) {
            throw new RangeError("Invalid iterator");
        }
        t = t.next();
        if (this.C === i) this.popFront(); else if (this.F === i) this.popBack(); else {
            const t = i.pre;
            const s = i.next;
            if (s) s.pre = t;
            if (t) t.next = s;
            this.o -= 1;
        }
        return t;
    }
    pushBack(t) {
        this.o += 1;
        const i = new LinkNode(t);
        if (!this.F) {
            this.C = this.F = i;
            this.L.next = this.C;
            this.C.pre = this.L;
        } else {
            this.F.next = i;
            i.pre = this.F;
            this.F = i;
        }
        this.F.next = this.L;
        this.L.pre = this.F;
    }
    popBack() {
        if (!this.F) return;
        this.o -= 1;
        if (this.C === this.F) {
            this.C = this.F = undefined;
            this.L.next = undefined;
        } else {
            this.F = this.F.pre;
            if (this.F) this.F.next = undefined;
        }
        this.L.pre = this.F;
        if (this.F) this.F.next = this.L;
    }
    setElementByPos(t, i) {
        if (t < 0 || t > this.o - 1) {
            throw new RangeError;
        }
        let s = this.C;
        while (t--) {
            s = s.next;
        }
        s.value = i;
    }
    insert(t, i, s = 1) {
        if (t < 0 || t > this.o) {
            throw new RangeError;
        }
        if (s <= 0) return;
        if (t === 0) {
            while (s--) this.pushFront(i);
        } else if (t === this.o) {
            while (s--) this.pushBack(i);
        } else {
            let e = this.C;
            for (let i = 1; i < t; ++i) {
                e = e.next;
            }
            const h = e.next;
            this.o += s;
            while (s--) {
                e.next = new LinkNode(i);
                e.next.pre = e;
                e = e.next;
            }
            e.next = h;
            if (h) h.pre = e;
        }
    }
    find(t) {
        if (!this.C) return this.end();
        let i = this.C;
        while (i !== this.L) {
            if (i.value === t) {
                return new LinkListIterator(i, this.L);
            }
            i = i.next;
        }
        return this.end();
    }
    reverse() {
        if (this.o <= 1) return;
        let t = this.C;
        let i = this.F;
        let s = 0;
        while (s << 1 < this.o) {
            const e = t.value;
            t.value = i.value;
            i.value = e;
            t = t.next;
            i = i.pre;
            s += 1;
        }
    }
    unique() {
        if (this.o <= 1) return;
        let t = this.C;
        while (t !== this.L) {
            let i = t;
            while (i.next && i.value === i.next.value) {
                i = i.next;
                this.o -= 1;
            }
            t.next = i.next;
            if (t.next) t.next.pre = t;
            t = t.next;
        }
    }
    sort(t) {
        if (this.o <= 1) return;
        const i = [];
        this.forEach((t => i.push(t)));
        i.sort(t);
        let s = this.C;
        i.forEach((t => {
            s.value = t;
            s = s.next;
        }));
    }
    pushFront(t) {
        this.o += 1;
        const i = new LinkNode(t);
        if (!this.C) {
            this.C = this.F = i;
            this.F.next = this.L;
            this.L.pre = this.F;
        } else {
            i.next = this.C;
            this.C.pre = i;
            this.C = i;
        }
        this.L.next = this.C;
        this.C.pre = this.L;
    }
    popFront() {
        if (!this.C) return;
        this.o -= 1;
        if (this.C === this.F) {
            this.C = this.F = undefined;
            this.L.pre = this.F;
        } else {
            this.C = this.C.next;
            if (this.C) this.C.pre = this.L;
        }
        this.L.next = this.C;
    }
    merge(t) {
        if (!this.C) {
            t.forEach((t => this.pushBack(t)));
            return;
        }
        let i = this.C;
        t.forEach((t => {
            while (i && i !== this.L && i.value <= t) {
                i = i.next;
            }
            if (i === this.L) {
                this.pushBack(t);
                i = this.F;
            } else if (i === this.C) {
                this.pushFront(t);
                i = this.C;
            } else {
                this.o += 1;
                const s = i.pre;
                s.next = new LinkNode(t);
                s.next.pre = s;
                s.next.next = i;
                i.pre = s.next;
            }
        }));
    }
    [Symbol.iterator]() {
        return function*() {
            if (!this.C) return;
            let t = this.C;
            while (t !== this.L) {
                yield t.value;
                t = t.next;
            }
        }.bind(this)();
    }
}

var _default = LinkList;

exports.default = _default;