"use strict";

Object.defineProperty(exports, "t", {
    value: true
});

exports.default = exports.VectorIterator = void 0;

var _Base = _interopRequireDefault(require("./Base"));

var _RandomIterator = require("./Base/RandomIterator");

function _interopRequireDefault(t) {
    return t && t.t ? t : {
        default: t
    };
}

class VectorIterator extends _RandomIterator.RandomIterator {
    copy() {
        return new VectorIterator(this.I, this.B, this.D, this.g, this.iteratorType);
    }
}

exports.VectorIterator = VectorIterator;

class Vector extends _Base.default {
    constructor(t = [], e = true) {
        super();
        if (Array.isArray(t)) {
            this.V = e ? [ ...t ] : t;
            this.o = t.length;
        } else {
            this.V = [];
            t.forEach((t => this.pushBack(t)));
        }
        this.size = this.size.bind(this);
        this.getElementByPos = this.getElementByPos.bind(this);
        this.setElementByPos = this.setElementByPos.bind(this);
    }
    clear() {
        this.o = 0;
        this.V.length = 0;
    }
    begin() {
        return new VectorIterator(0, this.size, this.getElementByPos, this.setElementByPos);
    }
    end() {
        return new VectorIterator(this.o, this.size, this.getElementByPos, this.setElementByPos);
    }
    rBegin() {
        return new VectorIterator(this.o - 1, this.size, this.getElementByPos, this.setElementByPos, 1);
    }
    rEnd() {
        return new VectorIterator(-1, this.size, this.getElementByPos, this.setElementByPos, 1);
    }
    front() {
        return this.V[0];
    }
    back() {
        return this.V[this.o - 1];
    }
    forEach(t) {
        for (let e = 0; e < this.o; ++e) {
            t(this.V[e], e);
        }
    }
    getElementByPos(t) {
        if (t < 0 || t > this.o - 1) {
            throw new RangeError;
        }
        return this.V[t];
    }
    eraseElementByPos(t) {
        if (t < 0 || t > this.o - 1) {
            throw new RangeError;
        }
        this.V.splice(t, 1);
        this.o -= 1;
    }
    eraseElementByValue(t) {
        let e = 0;
        for (let r = 0; r < this.o; ++r) {
            if (this.V[r] !== t) {
                this.V[e++] = this.V[r];
            }
        }
        this.o = this.V.length = e;
    }
    eraseElementByIterator(t) {
        const e = t.I;
        t = t.next();
        this.eraseElementByPos(e);
        return t;
    }
    pushBack(t) {
        this.V.push(t);
        this.o += 1;
    }
    popBack() {
        if (!this.o) return;
        this.V.pop();
        this.o -= 1;
    }
    setElementByPos(t, e) {
        if (t < 0 || t > this.o - 1) {
            throw new RangeError;
        }
        this.V[t] = e;
    }
    insert(t, e, r = 1) {
        if (t < 0 || t > this.o) {
            throw new RangeError;
        }
        this.V.splice(t, 0, ...new Array(r).fill(e));
        this.o += r;
    }
    find(t) {
        for (let e = 0; e < this.o; ++e) {
            if (this.V[e] === t) {
                return new VectorIterator(e, this.size, this.getElementByPos, this.getElementByPos);
            }
        }
        return this.end();
    }
    reverse() {
        this.V.reverse();
    }
    unique() {
        let t = 1;
        for (let e = 1; e < this.o; ++e) {
            if (this.V[e] !== this.V[e - 1]) {
                this.V[t++] = this.V[e];
            }
        }
        this.o = this.V.length = t;
    }
    sort(t) {
        this.V.sort(t);
    }
    [Symbol.iterator]() {
        return function*() {
            return yield* this.V;
        }.bind(this)();
    }
}

var _default = Vector;

exports.default = _default;