export declare const enum TreeNodeColor {
    RED = 1,
    BLACK = 0
}
export declare class TreeNode<K, V> {
    color: TreeNodeColor;
    key: K | undefined;
    value: V | undefined;
    left: TreeNode<K, V> | undefined;
    right: TreeNode<K, V> | undefined;
    parent: TreeNode<K, V> | undefined;
    constructor(key?: K, value?: V);
    /**
     * @description Get the pre node.
     * @return TreeNode about the pre node.
     */
    pre(): TreeNode<K, V>;
    /**
     * @description Get the next node.
     * @return TreeNode about the next node.
     */
    next(): TreeNode<K, V>;
    /**
     * @description Rotate left.
     * @return TreeNode about moved to original position after rotation.
     */
    rotateLeft(): TreeNode<K, V>;
    /**
     * @description Rotate right.
     * @return TreeNode about moved to original position after rotation.
     */
    rotateRight(): TreeNode<K, V>;
}
export declare class TreeNodeEnableIndex<K, V> extends TreeNode<K, V> {
    left: TreeNodeEnableIndex<K, V> | undefined;
    right: TreeNodeEnableIndex<K, V> | undefined;
    parent: TreeNodeEnableIndex<K, V> | undefined;
    subTreeSize: number;
    /**
     * @description Rotate left and do recount.
     * @return TreeNode about moved to original position after rotation.
     */
    rotateLeft(): TreeNodeEnableIndex<K, V>;
    /**
     * @description Rotate right and do recount.
     * @return TreeNode about moved to original position after rotation.
     */
    rotateRight(): TreeNode<K, V>;
    recount(): void;
}
