import type TreeIterator from './TreeIterator';
import { Container } from "../../ContainerBase";
declare abstract class TreeContainer<K, V> extends Container<K | [K, V]> {
    protected constructor(_cmp?: (x: K, y: K) => number, enableIndex?: boolean);
    /**
     * @param key The given key you want to compare.
     * @return An iterator to the first element not less than the given key.
     */
    abstract lowerBound(key: K): TreeIterator<K, V>;
    /**
     * @param key The given key you want to compare.
     * @return An iterator to the first element greater than the given key.
     */
    abstract upperBound(key: K): TreeIterator<K, V>;
    /**
     * @param key The given key you want to compare.
     * @return An iterator to the first element not greater than the given key.
     */
    abstract reverseLowerBound(key: K): TreeIterator<K, V>;
    /**
     * @param key The given key you want to compare.
     * @return An iterator to the first element less than the given key.
     */
    abstract reverseUpperBound(key: K): TreeIterator<K, V>;
    /**
     * @description Union the other tree to self.
     *              <br/>
     *              Waiting for optimization, this is O(mlog(n+m)) algorithm now,
     *              but we expect it to be O(mlog(n/m+1)).<br/>
     *              More information =>
     *              https://en.wikipedia.org/wiki/Red_black_tree
     *              <br/>
     * @param other The other tree container you want to merge.
     */
    abstract union(other: TreeContainer<K, V>): void;
    clear(): void;
    /**
     * @description Update node's key by iterator.
     * @param iter The iterator you want to change.
     * @param key The key you want to update.
     * @return Boolean about if the modification is successful.
     */
    updateKeyByIterator(iter: TreeIterator<K, V>, key: K): boolean;
    eraseElementByPos(pos: number): void;
    /**
     * @description Remove the element of the specified key.
     * @param key The key you want to remove.
     */
    eraseElementByKey(key: K): void;
    eraseElementByIterator(iter: TreeIterator<K, V>): TreeIterator<K, V>;
    /**
     * @description Get the height of the tree.
     * @return Number about the height of the RB-tree.
     */
    getHeight(): number;
}
export default TreeContainer;
